@@ -19,7 +19,7 @@ package MongoDB::BulkWrite;
 # ABSTRACT: MongoDB bulk write interface
 
 use version;
-our $VERSION = 'v0.999.998.5';
+our $VERSION = 'v0.999.998.6'; # TRIAL
 
 use MongoDB::Error;
 use MongoDB::OID;
@@ -157,14 +157,34 @@ sub insert {
         MongoDB::UsageError->throw("argument to insert must be a single hashref, arrayref or Tie::IxHash");
     }
 
+    # clone docs to add id and leave original unmodified
     if ( ref $doc eq 'ARRAY' ) {
         MongoDB::UsageError->throw("array reference to insert must have key/value pairs")
           if @$doc % 2;
         $doc = Tie::IxHash->new(@$doc);
     }
+    elsif ( ref $doc eq 'HASH' ) {
+        $doc = Tie::IxHash->new(%$doc);
+    }
+
+    if ( $doc->Length && $doc->Keys(0) eq '_id' ) {
+        $self->_enqueue_write( [ insert => $doc ] );
+    }
+    else {
+        # XXX this encapsulation violation is ugly, but a lot faster than
+        # doing it element by element.
+        my $copy = bless [
+            { %{ $doc->[0] } },
+            [ @{ $doc->[1] } ],
+            [ @{ $doc->[2] } ],
+            0
+        ], "Tie::IxHash";
+        my $id = $copy->DELETE('_id');
+        $id = MongoDB::OID->new unless defined $id;
+        $copy->Unshift( _id => $id );
+        $self->_enqueue_write( [ insert => $copy ] );
+    }
 
-    $self->collection->_add_oids([$doc]);
-    $self->_enqueue_write( [ insert => $doc ] );
     return;
 }
 
@@ -226,6 +246,7 @@ sub execute {
         coll_name     => $self->collection->name,
         queue         => $self->_queue,
         ordered       => $self->ordered,
+        bson_codec    => $self->collection->bson_codec,
         write_concern => $write_concern,
     );
 
@@ -249,7 +270,7 @@ MongoDB::BulkWrite - MongoDB bulk write interface
 
 =head1 VERSION
 
-version v0.999.998.5
+version v0.999.998.6
 
 =head1 SYNOPSIS
 
