@@ -19,7 +19,7 @@ package MongoDB::BulkWriteResult;
 # ABSTRACT: MongoDB bulk write result document
 
 use version;
-our $VERSION = 'v0.999.998.4'; # TRIAL
+our $VERSION = 'v0.999.998.6'; # TRIAL
 
 # empty superclass for backcompatibility; add a variable to the
 # package namespace so Perl thinks it's a real package
@@ -142,18 +142,18 @@ sub _parse_cmd_result {
     my $args = ref $_[0] eq 'HASH' ? shift : {@_};
 
     unless ( 2 == grep { exists $args->{$_} } qw/op result/ ) {
-        confess "parse requires 'op' and 'result' arguments";
+        MongoDB::UsageError->throw("parse requires 'op' and 'result' arguments");
     }
 
     my ( $op, $op_count, $batch_count, $result, $cmd_doc ) =
       @{$args}{qw/op op_count batch_count result cmd_doc/};
 
-    $result = $result->result
+    $result = $result->output
       if eval { $result->isa("MongoDB::CommandResult") };
 
-    confess "op argument to parse must be one of: @op_map_keys"
+    MongoDB::UsageError->throw("op argument to parse must be one of: @op_map_keys")
       unless $op eq any(@op_map_keys);
-    confess "results argument to parse must be a hash reference"
+    MongoDB::UsageError->throw("results argument to parse must be a hash reference")
       unless ref $result eq 'HASH';
 
     my $attrs = {
@@ -310,7 +310,7 @@ MongoDB::BulkWriteResult - MongoDB bulk write result document
 
 =head1 VERSION
 
-version v0.999.998.4
+version v0.999.998.6
 
 =head1 SYNOPSIS
 
