@@ -19,11 +19,12 @@ package MongoDB::BulkWriteView;
 # ABSTRACT: Bulk write operations against a query document
 
 use version;
-our $VERSION = 'v0.999.998.4'; # TRIAL
+our $VERSION = 'v0.999.998.6'; # TRIAL
 
 use Moose;
 
 use Syntax::Keyword::Junction qw/any/;
+use MongoDB::Error;
 use MongoDB::_Types -types;
 use Types::Standard -types;
 use boolean;
@@ -54,7 +55,7 @@ has _upsert => (
 sub upsert {
     my ($self) = @_;
     unless ( @_ == 1 ) {
-        confess "the upsert method takes no arguments";
+        MongoDB::UsageError->throw("the upsert method takes no arguments");
     }
     return $self->new( %$self, _upsert => true );
 }
@@ -79,11 +80,11 @@ sub _update {
     my ( $self, $doc ) = @_;
 
     unless ( @_ == 2 && ref $doc eq any(qw/HASH ARRAY Tie::IxHash/) ) {
-        confess "argument to $method must be a single hashref, arrayref or Tie::IxHash";
+        MongoDB::UsageError->throw("argument to $method must be a single hashref, arrayref or Tie::IxHash");
     }
 
     if ( ref $doc eq 'ARRAY' ) {
-        confess "array reference to $method must have key/value pairs"
+        MongoDB::UsageError->throw("array reference to $method must have key/value pairs")
           if @$doc % 2;
         $doc = Tie::IxHash->new(@$doc);
     }
@@ -91,23 +92,12 @@ sub _update {
         $doc = Tie::IxHash->new(%$doc);
     }
 
-    my @keys = $doc->Keys;
-    if ( $method eq 'replace_one' ) {
-        if ( my @bad = grep { substr( $_, 0, 1 ) eq '$' } @keys ) {
-            confess "$method document can't have '\$' prefixed field names: @bad";
-        }
-    }
-    else {
-        if ( my @bad = grep { substr( $_, 0, 1 ) ne '$' } @keys ) {
-            confess "$method document can't have non- '\$' prefixed field names: @bad";
-        }
-    }
-
     my $update = {
         q      => $self->_query,
         u      => $doc,
         multi  => $method eq 'update' ? true : false,
         upsert => boolean( $self->_upsert ),
+        is_replace => $method eq 'replace_one',
     };
 
     $self->_enqueue_write( [ update => $update ] );
@@ -143,7 +133,7 @@ MongoDB::BulkWriteView - Bulk write operations against a query document
 
 =head1 VERSION
 
-version v0.999.998.4
+version v0.999.998.6
 
 =head1 SYNOPSIS
 
