@@ -20,7 +20,7 @@ package MongoDB::Collection;
 # ABSTRACT: A MongoDB Collection
 
 use version;
-our $VERSION = 'v0.999.998.4'; # TRIAL
+our $VERSION = 'v0.999.998.6'; # TRIAL
 
 use MongoDB::Error;
 use MongoDB::InsertManyResult;
@@ -108,6 +108,19 @@ has write_concern => (
     coerce   => 1,
 );
 
+#pod =attr bson_codec
+#pod
+#pod An object that provides the C<encode_one> and C<decode_one> methods, such
+#pod as from L<MongoDB::BSON>.
+#pod
+#pod =cut
+
+has bson_codec => (
+    is       => 'ro',
+    isa      => BSONCodec,
+    required => 1,
+);
+
 #--------------------------------------------------------------------------#
 # computed attributes
 #--------------------------------------------------------------------------#
@@ -176,19 +189,63 @@ sub clone {
     my ($self, @args) = @_;
     my $class = ref($self);
     if ( @args == 1 && ref( $args[0] ) eq 'HASH' ) {
-        return class->new( %$self, %{$args[0]} );
+        return $class->new( %$self, %{$args[0]} );
     }
 
     return $class->new( %$self, @args );
 }
 
+#pod =method with_codec
+#pod
+#pod     $coll2 = $coll1->with_codec( $new_codec );
+#pod     $coll2 = $coll1->with_codec( prefer_numeric => 1 );
+#pod
+#pod Constructs a copy of the original collection, but clones the C<bson_codec>.
+#pod If given an object that does C<encode_one> and C<decode_one>, it is
+#pod equivalent to:
+#pod
+#pod     $coll2 = $coll1->clone( bson_codec => $new_codec );
+#pod
+#pod If given a hash reference or a list of key/value pairs, it is equivalent
+#pod to:
+#pod
+#pod     $coll2 = $coll1->clone(
+#pod         bson_codec => $coll1->bson_codec->clone( @list )
+#pod     );
+#pod
+#pod =cut
+
+sub with_codec {
+    my ( $self, @args ) = @_;
+    if ( @args == 1 ) {
+        my $arg = $args[0];
+        if ( eval { $arg->can('encode_bson') && $arg->can('decode_bson') } ) {
+            return $self->clone( bson_codec => $arg );
+        }
+        elsif ( ref $arg eq 'HASH' ) {
+            return $self->clone( bson_codec => $self->bson_codec->clone(%$arg) );
+        }
+    }
+    elsif ( @args % 2 == 0 ) {
+        return $self->clone( bson_codec => $self->bson_codec->clone(@args) );
+    }
+
+    # fallthrough is argument error
+    MongoDB::UsageError->throw(
+        "argument to with_codec must be new codec, hashref or key/value pairs" );
+}
+
 #pod =method insert_one
 #pod
 #pod     $res = $coll->insert_one( $document );
+#pod     $id = $res->inserted_id;
 #pod
 #pod Inserts a single L<document|/Document> into the database and returns a
-#pod L<MongoDB::InsertOneResult> object.If no C<_id> field is present, one
-#pod will be added to the original document.
+#pod L<MongoDB::InsertOneResult> object.
+#pod
+#pod If no C<_id> field is present, one will be added when a document is
+#pod serialized for the database without modifying the original document.
+#pod The generated C<_id> may be retrieved from the result object.
 #pod
 #pod =cut
 
@@ -197,10 +254,9 @@ sub insert_one {
     $insert_one_args ||= compile( Object, IxHash);
     my ( $self, $document ) = $insert_one_args->(@_);
 
-    $self->_add_oids( [$document] );
-
     my $op = MongoDB::Op::_InsertOne->new(
         db_name       => $self->database->name,
+        bson_codec    => $self->bson_codec,
         coll_name     => $self->name,
         document      => $document,
         write_concern => $self->write_concern,
@@ -218,8 +274,9 @@ sub insert_one {
 #pod database and returns a L<MongoDB::InsertManyResult>.  This is syntactic sugar
 #pod for doing a L<MongoDB::BulkWrite> operation.
 #pod
-#pod If no C<_id> field is present in a document, one will be added to the original
-#pod document.
+#pod If no C<_id> field is present, one will be added when a document is
+#pod serialized for the database without modifying the original document.
+#pod The generated C<_id> may be retrieved from the result object.
 #pod
 #pod An optional hash reference of options may be provided.  The only valid option
 #pod is C<ordered>, which defaults to true.  When true, the server will halt
@@ -270,6 +327,7 @@ sub delete_one {
     my $op = MongoDB::Op::_Delete->new(
         db_name       => $self->database->name,
         coll_name     => $self->name,
+        bson_codec    => $self->bson_codec,
         filter        => $filter,
         just_one      => 1,
         write_concern => $self->write_concern,
@@ -297,6 +355,7 @@ sub delete_many {
     my $op = MongoDB::Op::_Delete->new(
         db_name       => $self->database->name,
         coll_name     => $self->name,
+        bson_codec    => $self->bson_codec,
         filter        => $filter,
         just_one      => 0,
         write_concern => $self->write_concern,
@@ -325,16 +384,18 @@ sub delete_many {
 
 my $replace_one_args;
 sub replace_one {
-    $replace_one_args ||= compile( Object, IxHash, ReplaceDoc, Optional[HashRef] );
+    $replace_one_args ||= compile( Object, IxHash, IxHash, Optional[HashRef] );
     my ($self, $filter, $replacement, $options) = $replace_one_args->(@_);
 
     my $op = MongoDB::Op::_Update->new(
         db_name       => $self->database->name,
         coll_name     => $self->name,
+        bson_codec    => $self->bson_codec,
         filter        => $filter,
         update        => $replacement,
         multi         => false,
         upsert        => $options->{upsert} ? true : false,
+        is_replace    => 1,
         write_concern => $self->write_concern,
     );
 
@@ -361,16 +422,18 @@ sub replace_one {
 
 my $update_one_args;
 sub update_one {
-    $update_one_args ||= compile( Object, IxHash, UpdateDoc, Optional[HashRef] );
+    $update_one_args ||= compile( Object, IxHash, IxHash, Optional[HashRef] );
     my ($self, $filter, $update, $options) = $update_one_args->(@_);
 
     my $op = MongoDB::Op::_Update->new(
         db_name       => $self->database->name,
         coll_name     => $self->name,
+        bson_codec    => $self->bson_codec,
         filter        => $filter,
         update        => $update,
         multi         => false,
         upsert        => $options->{upsert} ? true : false,
+        is_replace    => 0,
         write_concern => $self->write_concern,
     );
 
@@ -397,16 +460,18 @@ sub update_one {
 
 my $update_many_args;
 sub update_many {
-    $update_many_args ||= compile( Object, IxHash, UpdateDoc, Optional[HashRef] );
+    $update_many_args ||= compile( Object, IxHash, IxHash, Optional[HashRef] );
     my ($self, $filter, $update, $options) = $update_many_args->(@_);
 
     my $op = MongoDB::Op::_Update->new(
         db_name       => $self->database->name,
         coll_name     => $self->name,
+        bson_codec    => $self->bson_codec,
         filter        => $filter,
         update        => $update,
         multi         => true,
         upsert        => $options->{upsert} ? true : false,
+        is_replace    => 0,
         write_concern => $self->write_concern,
     );
 
@@ -489,6 +554,7 @@ sub find {
         %$options,
         db_name         => $self->database->name,
         coll_name       => $self->name,
+        bson_codec      => $self->bson_codec,
         client          => $self->client,
         read_preference => $self->read_preference,
         filter          => $filter || {},
@@ -548,6 +614,7 @@ sub find_one {
         %$options,
         db_name         => $self->database->name,
         coll_name       => $self->name,
+        bson_codec      => $self->bson_codec,
         client          => $self->client,
         read_preference => $self->read_preference,
         filter          => $filter || {},
@@ -636,7 +703,7 @@ sub find_one_and_delete {
 
 my $foar_args;
 sub find_one_and_replace {
-    $foar_args ||= compile( Object, IxHash, ReplaceDoc, Optional[HashRef] );
+    $foar_args ||= compile( Object, IxHash, IxHash, Optional[HashRef] );
     my ( $self, $filter, $replacement, $options ) = $foar_args->(@_);
 
     return $self->_find_one_and_update_or_replace($filter, $replacement, $options);
@@ -675,7 +742,7 @@ sub find_one_and_replace {
 
 my $foau_args;
 sub find_one_and_update {
-    $foau_args ||= compile( Object, IxHash, UpdateDoc, Optional[HashRef] );
+    $foau_args ||= compile( Object, IxHash, IxHash, Optional[HashRef] );
     my ( $self, $filter, $update, $options ) = $foau_args->(@_);
 
     return $self->_find_one_and_update_or_replace($filter, $update, $options);
@@ -736,7 +803,7 @@ sub aggregate {
         db_name    => $self->database->name,
         coll_name  => $self->name,
         client     => $self->client,
-        bson_codec => $self->client,
+        bson_codec => $self->bson_codec,
         pipeline   => $pipeline,
         options    => $options,
         ( $read_pref ? ( read_preference => $read_pref ) : () ),
@@ -825,7 +892,7 @@ sub distinct {
         db_name         => $self->database->name,
         coll_name       => $self->name,
         client          => $self->client,
-        bson_codec      => $self->client,
+        bson_codec      => $self->bson_codec,
         fieldname       => $fieldname,
         filter          => $filter,
         options         => $options,
@@ -868,11 +935,12 @@ sub parallel_scan {
     my $op = MongoDB::Op::_Command->new(
         db_name         => $db->name,
         query           => \@command,
+        bson_codec      => $self->bson_codec,
         read_preference => $self->read_preference,
     );
 
     my $result = $self->client->send_read_op( $op );
-    my $response = $result->result;
+    my $response = $result->output;
 
     Carp::croak("No cursors returned")
         unless $response->{cursors} && ref $response->{cursors} eq 'ARRAY';
@@ -880,9 +948,10 @@ sub parallel_scan {
     my @cursors;
     for my $c ( map { $_->{cursor} } @{$response->{cursors}} ) {
         my $qr = MongoDB::QueryResult->new(
-            _client => $self->client,
-            address => $result->address,
-            cursor  => $c,
+            _client    => $self->client,
+            address    => $result->address,
+            bson_codec => $self->bson_codec,
+            cursor     => $c,
         );
         push @cursors, $qr;
     }
@@ -949,7 +1018,7 @@ sub rename {
 
 sub ensure_index {
     my ( $self, $keys, $opts ) = @_;
-    MongoDB::Error->throw("ensure_index options must be a hash reference")
+    MongoDB::UsageError->throw("ensure_index options must be a hash reference")
       if $opts && !ref($opts) eq 'HASH';
 
     $keys = Tie::IxHash->new(@$keys) if ref $keys eq 'ARRAY';
@@ -964,6 +1033,7 @@ sub ensure_index {
     my $op = MongoDB::Op::_CreateIndexes->new(
         db_name       => $self->database->name,
         coll_name     => $self->name,
+        bson_codec    => $self->bson_codec,
         indexes       => [ { key => $keys, %$opts } ],
         write_concern => $wc,
     );
@@ -1096,7 +1166,7 @@ sub get_indexes {
         db_name    => $self->database->name,
         coll_name  => $self->name,
         client     => $self->client,
-        bson_codec => $self->client,
+        bson_codec => $self->bson_codec,
     );
 
     my $res = $self->client->send_read_op($op);
@@ -1216,9 +1286,10 @@ sub initialize_unordered_bulk_op {
 sub bulk_write {
     my ( $self, $requests, $options ) = @_;
 
-    confess 'requests not an array reference' unless ref $requests eq 'ARRAY';
-    confess 'empty request list' unless @$requests;
-    confess 'options not a hash reference'
+    MongoDB::UsageError->throw("requests not an array reference")
+      unless ref $requests eq 'ARRAY';
+    MongoDB::UsageError->throw("empty request list") unless @$requests;
+    MongoDB::UsageError->throw("options not a hash reference")
       if defined($options) && ref($options) ne 'HASH';
 
     $options ||= { ordered => 1 };
@@ -1239,7 +1310,7 @@ sub bulk_write {
                 ( $method, $args ) = %{ $requests->[$i] };
             }
             else {
-                confess "$requests->[$i] is not a hash or array reference";
+                MongoDB::UsageError->throw("$requests->[$i] is not a hash or array reference");
             }
             $i++;
         }
@@ -1248,7 +1319,7 @@ sub bulk_write {
             $i += 2;
         }
 
-        confess "'$method' requires an array reference of arguments"
+        MongoDB::UsageError->throw("'$method' requires an array reference of arguments")
           unless ref($args) eq 'ARRAY';
 
         # handle inserts
@@ -1284,7 +1355,7 @@ sub bulk_write {
                 $view->update($doc);
             }
             else {
-                confess "unknown bulk operation '$method'";
+                MongoDB::UsageError->throw("unknown bulk operation '$method'");
             }
         }
     }
@@ -1304,48 +1375,6 @@ BEGIN {
 # private methods
 #--------------------------------------------------------------------------#
 
-sub _add_oids {
-    my ($self, $target) = @_;
-    my @ids;
-
-    for my $d ( ref($target) eq 'ARRAY' ? @$target : $target ) {
-        my $type = reftype($d);
-        my $found_id;
-        if (ref($d) eq 'Tie::IxHash') {
-            $found_id = $d->FETCH('_id');
-            unless ( defined $found_id ) {
-                $d->Unshift( '_id', $found_id = MongoDB::OID->new );
-            }
-        }
-        elsif ($type eq 'ARRAY') {
-            # search for an _id or prepend one
-            for my $i ( 0 .. (@$d/2 - 1) ) {
-                if ( $d->[2*$i] eq '_id' ) {
-                    $found_id = $d->[2*$i+1];
-                    last;
-                }
-            }
-            unless (defined $found_id) {
-                unshift @$d, '_id', $found_id = MongoDB::OID->new;
-            }
-        }
-        elsif ($type eq 'HASH') {
-            $found_id = $d->{_id};
-            unless ( defined $found_id ) {
-                $found_id = MongoDB::OID->new;
-                $d->{_id} = $found_id;
-            }
-        }
-        else {
-            $type = 'scalar' unless $type;
-            Carp::croak("unhandled type $type")
-        }
-        push @ids, $found_id;
-    }
-
-    return \@ids;
-}
-
 sub _dynamic_write_concern {
     my ( $self, $opts ) = @_;
     if ( !exists( $opts->{safe} ) || $opts->{safe} ) {
@@ -1367,7 +1396,7 @@ sub _find_one_and_update_or_replace {
 
     # returnDocument ('before'|'after') maps to field 'new'
     if ( exists $options->{returnDocument} ) {
-        confess "Invalid returnDocument parameter '$options->{returnDocument}'"
+        MongoDB::UsageError->throw("Invalid returnDocument parameter '$options->{returnDocument}'")
             unless $options->{returnDocument} =~ /^(?:before|after)$/;
         $options->{new} = delete( $options->{returnDocument} ) eq 'after' ? true : false;
     }
@@ -1385,6 +1414,7 @@ sub _find_one_and_update_or_replace {
 sub _try_find_and_modify {
     my ($self, $command) = @_;
     my $result;
+    # XXX this ought to use our codec
     try {
         $result = $self->database->run_command( $command );
     }
@@ -1448,7 +1478,7 @@ sub __ixhash {
         $hash->{$key} = Tie::IxHash->new( @$ref );
     }
     else {
-        confess "Can't convert $type to a Tie::IxHash";
+        MongoDB::UsageError->throw("Can't convert $type to a Tie::IxHash");
     }
     return;
 }
@@ -1474,7 +1504,7 @@ sub __to_index_string {
         }
     }
     else {
-        confess 'expected Tie::IxHash, hash, or array reference for keys';
+        MongoDB::UsageError->throw("expected Tie::IxHash, hash, or array reference for keys");
     }
 
     return join("_", @name);
@@ -1489,13 +1519,10 @@ sub insert {
     $legacy_insert_args ||= compile( Object, IxHash, Optional[HashRef] );
     my ( $self, $document, $opts ) = $legacy_insert_args->(@_);
 
-    unless ( $opts->{'no_ids'} ) {
-        $self->_add_oids( $document );
-    }
-
     my $op = MongoDB::Op::_InsertOne->new(
         db_name       => $self->database->name,
         coll_name     => $self->name,
+        bson_codec    => $self->bson_codec,
         document      => $document,
         write_concern => $self->_dynamic_write_concern($opts),
     );
@@ -1510,13 +1537,10 @@ sub batch_insert {
     my ( $self, $documents, $opts ) = @_;
     $legacy_batch_args ||= compile( Object, ArrayRef[IxHash], Optional[HashRef] );
 
-    unless ( $opts->{'no_ids'} ) {
-        $self->_add_oids($documents);
-    }
-
     my $op = MongoDB::Op::_BatchInsert->new(
         db_name       => $self->database->name,
         coll_name     => $self->name,
+        bson_codec    => $self->bson_codec,
         documents     => $documents,
         write_concern => $self->_dynamic_write_concern($opts),
     );
@@ -1541,6 +1565,7 @@ sub remove {
     my $op = MongoDB::Op::_Delete->new(
         db_name       => $self->database->name,
         coll_name     => $self->name,
+        bson_codec    => $self->bson_codec,
         filter        => $query || {},
         just_one      => !! $opts->{just_one},
         write_concern => $self->_dynamic_write_concern($opts),
@@ -1563,19 +1588,29 @@ sub update {
 
     if ( exists $opts->{multiple} ) {
         if ( exists( $opts->{multi} ) && !!$opts->{multi} ne !!$opts->{multiple} ) {
-            MongoDB::Error->throw(
+            MongoDB::UsageError->throw(
                 "can't use conflicting values of 'multiple' and 'multi' in 'update'");
         }
         $opts->{multi} = delete $opts->{multiple};
     }
 
+    # figure out if first key based on op_char or '$'
+    my $is_replace;
+    if ( $object && $object->Length ) {
+        my $fk = substr($object->FIRSTKEY, 0, 1);
+        my $op_char = eval { $self->bson_codec->op_char } || '';
+        $is_replace = $fk ne '$' && $fk ne $op_char;
+    }
+
     my $op = MongoDB::Op::_Update->new(
         db_name       => $self->database->name,
         coll_name     => $self->name,
+        bson_codec    => $self->bson_codec,
         filter        => $query || {},
         update        => $object || {},
         multi         => $opts->{multi},
         upsert        => $opts->{upsert},
+        is_replace    => $is_replace,
         write_concern => $self->_dynamic_write_concern($opts),
     );
 
@@ -1630,7 +1665,7 @@ MongoDB::Collection - A MongoDB Collection
 
 =head1 VERSION
 
-version v0.999.998.4
+version v0.999.998.6
 
 =head1 SYNOPSIS
 
@@ -1742,6 +1777,11 @@ A L<MongoDB::WriteConcern> object.  It may be initialized with a hash
 reference that will be coerced into a new MongoDB::WriteConcern object.
 By default it will be inherited from a L<MongoDB::Database> object.
 
+=head2 bson_codec
+
+An object that provides the C<encode_one> and C<decode_one> methods, such
+as from L<MongoDB::BSON>.
+
 =head1 METHODS
 
 =head2 client
@@ -1766,13 +1806,35 @@ database "test" would result in a C<full_name> of "test.foo".
 Constructs a copy of the original collection, but allows changing
 attributes in the copy.
 
+=head2 with_codec
+
+    $coll2 = $coll1->with_codec( $new_codec );
+    $coll2 = $coll1->with_codec( prefer_numeric => 1 );
+
+Constructs a copy of the original collection, but clones the C<bson_codec>.
+If given an object that does C<encode_one> and C<decode_one>, it is
+equivalent to:
+
+    $coll2 = $coll1->clone( bson_codec => $new_codec );
+
+If given a hash reference or a list of key/value pairs, it is equivalent
+to:
+
+    $coll2 = $coll1->clone(
+        bson_codec => $coll1->bson_codec->clone( @list )
+    );
+
 =head2 insert_one
 
     $res = $coll->insert_one( $document );
+    $id = $res->inserted_id;
 
 Inserts a single L<document|/Document> into the database and returns a
-L<MongoDB::InsertOneResult> object.If no C<_id> field is present, one
-will be added to the original document.
+L<MongoDB::InsertOneResult> object.
+
+If no C<_id> field is present, one will be added when a document is
+serialized for the database without modifying the original document.
+The generated C<_id> may be retrieved from the result object.
 
 =head2 insert_many
 
@@ -1783,8 +1845,9 @@ Inserts each of the L<documents|/Documents> in an array reference into the
 database and returns a L<MongoDB::InsertManyResult>.  This is syntactic sugar
 for doing a L<MongoDB::BulkWrite> operation.
 
-If no C<_id> field is present in a document, one will be added to the original
-document.
+If no C<_id> field is present, one will be added when a document is
+serialized for the database without modifying the original document.
+The generated C<_id> may be retrieved from the result object.
 
 An optional hash reference of options may be provided.  The only valid option
 is C<ordered>, which defaults to true.  When true, the server will halt
@@ -2481,4 +2544,4 @@ This is free software, licensed under:
 __END__
 
 
-# vim: ts=4 sts=4 sw=4 et:
+# vim: set ts=4 sts=4 sw=4 et tw=75:
