@@ -20,7 +20,7 @@ package MongoDB::Database;
 # ABSTRACT: A MongoDB Database
 
 use version;
-our $VERSION = 'v0.999.998.4'; # TRIAL
+our $VERSION = 'v0.999.998.6'; # TRIAL
 
 use MongoDB::CommandResult;
 use MongoDB::Error;
@@ -84,6 +84,19 @@ has write_concern => (
     coerce   => 1,
 );
 
+#pod =attr bson_codec
+#pod
+#pod An object that provides the C<encode_one> and C<decode_one> methods, such
+#pod as from L<MongoDB::BSON>.
+#pod
+#pod =cut
+
+has bson_codec => (
+    is       => 'ro',
+    isa      => BSONCodec,
+    required => 1,
+);
+
 #--------------------------------------------------------------------------#
 # methods
 #--------------------------------------------------------------------------#
@@ -102,7 +115,7 @@ sub collection_names {
     my $op = MongoDB::Op::_ListCollections->new(
         db_name    => $self->name,
         client     => $self->_client,
-        bson_codec => $self->_client,
+        bson_codec => $self->bson_codec,
     );
 
     my $res = $self->_client->send_read_op($op);
@@ -131,6 +144,7 @@ sub get_collection {
     return MongoDB::Collection->new(
         read_preference => $self->read_preference,
         write_concern   => $self->write_concern,
+        bson_codec      => $self->bson_codec,
         ( $options ? %$options : () ),
         # not allowed to be overridden by options
         database => $self,
@@ -186,9 +200,9 @@ sub drop {
 
 #pod =method run_command
 #pod
-#pod     my $result = $database->run_command([ some_command => 1 ]);
+#pod     my $output = $database->run_command([ some_command => 1 ]);
 #pod
-#pod     my $result = $database->run_command(
+#pod     my $output = $database->run_command(
 #pod         [ some_command => 1 ],
 #pod         { mode => 'secondaryPreferred' }
 #pod     );
@@ -204,7 +218,7 @@ sub drop {
 #pod be a L<MongoDB::ReadPreference> object or a hash reference that can be used to
 #pod construct one.
 #pod
-#pod It returns the result of the command (a hash reference) on success or throws a
+#pod It returns the output of the command (a hash reference) on success or throws a
 #pod L<MongoDB::DatabaseError|MongoDB::Error/MongoDB::DatabaseError> exception if
 #pod the command fails.
 #pod
@@ -228,17 +242,18 @@ sub run_command {
     my $op = MongoDB::Op::_Command->new(
         db_name         => $self->name,
         query           => $command,
+        bson_codec      => $self->bson_codec,
         ( $read_pref ? ( read_preference => $read_pref ) : () ),
     );
 
     my $obj = $self->_client->send_read_op($op);
 
-    return $obj->result;
+    return $obj->output;
 }
 
 #pod =method eval ($code, $args?, $nolock?)
 #pod
-#pod     my $result = $database->eval('function(x) { return "hello, "+x; }', ["world"]);
+#pod     my $output = $database->eval('function(x) { return "hello, "+x; }', ["world"]);
 #pod
 #pod Evaluate a JavaScript expression on the Mongo server. The C<$code> argument can
 #pod be a string or an instance of L<MongoDB::Code>.  The C<$args> are an optional
@@ -265,12 +280,12 @@ sub eval {
              'args' => $args,
              'nolock' => $nolock);
 
-    my $result = $self->run_command($cmd);
-    if (ref $result eq 'HASH' && exists $result->{'retval'}) {
-        return $result->{'retval'};
+    my $output = $self->run_command($cmd);
+    if (ref $output eq 'HASH' && exists $output->{'retval'}) {
+        return $output->{'retval'};
     }
     else {
-        return $result;
+        return $output;
     }
 }
 
@@ -299,7 +314,7 @@ MongoDB::Database - A MongoDB Database
 
 =head1 VERSION
 
-version v0.999.998.4
+version v0.999.998.6
 
 =head1 SYNOPSIS
 
@@ -367,6 +382,11 @@ A L<MongoDB::WriteConcern> object.  It may be initialized with a hash
 reference that will be coerced into a new MongoDB::WriteConcern object.
 By default it will be inherited from a L<MongoDB::MongoClient> object.
 
+=head2 bson_codec
+
+An object that provides the C<encode_one> and C<decode_one> methods, such
+as from L<MongoDB::BSON>.
+
 =head1 METHODS
 
 =head2 collection_names
@@ -412,9 +432,9 @@ Deletes the database.
 
 =head2 run_command
 
-    my $result = $database->run_command([ some_command => 1 ]);
+    my $output = $database->run_command([ some_command => 1 ]);
 
-    my $result = $database->run_command(
+    my $output = $database->run_command(
         [ some_command => 1 ],
         { mode => 'secondaryPreferred' }
     );
@@ -430,7 +450,7 @@ second argument may specify an alternative read preference.  If given, it must
 be a L<MongoDB::ReadPreference> object or a hash reference that can be used to
 construct one.
 
-It returns the result of the command (a hash reference) on success or throws a
+It returns the output of the command (a hash reference) on success or throws a
 L<MongoDB::DatabaseError|MongoDB::Error/MongoDB::DatabaseError> exception if
 the command fails.
 
@@ -444,7 +464,7 @@ on database commands: L<http://dochub.mongodb.org/core/commands>.
 
 =head2 eval ($code, $args?, $nolock?)
 
-    my $result = $database->eval('function(x) { return "hello, "+x; }', ["world"]);
+    my $output = $database->eval('function(x) { return "hello, "+x; }', ["world"]);
 
 Evaluate a JavaScript expression on the Mongo server. The C<$code> argument can
 be a string or an instance of L<MongoDB::Code>.  The C<$args> are an optional
