@@ -22,7 +22,7 @@ package MongoDB::Error;
 # ABSTRACT: MongoDB Driver Error classes
 
 use version;
-our $VERSION = 'v0.999.998.5';
+our $VERSION = 'v0.999.998.6'; # TRIAL
 
 use Moose;
 use Carp;
@@ -128,7 +128,7 @@ extends("MongoDB::Error");
 
 has document => (
     is       => 'ro',
-    isa      => HashRef|IxHash,
+    isa      => Any,
     required => 1,
 );
 
@@ -229,6 +229,9 @@ Moose::Meta::Class->create( __PACKAGE__, superclasses => ['MongoDB::Error'] );
 package MongoDB::CursorNotFoundError;
 Moose::Meta::Class->create( __PACKAGE__, superclasses => ['MongoDB::Error'] );
 
+package MongoDB::DecodingError;
+Moose::Meta::Class->create( __PACKAGE__, superclasses => ['MongoDB::Error'] );
+
 package MongoDB::GridFSError;
 Moose::Meta::Class->create( __PACKAGE__, superclasses => ['MongoDB::Error'] );
 
@@ -269,7 +272,7 @@ MongoDB::Error - MongoDB Driver Error classes
 
 =head1 VERSION
 
-version v0.999.998.5
+version v0.999.998.6
 
 =head1 SYNOPSIS
 
@@ -343,6 +346,8 @@ To retry failures automatically, consider using L<Try::Tiny::Retry>.
         |   |
         |   |->MongoDB::WriteConcernError
         |
+        |->MongoDB::DecodingError
+        |
         |->MongoDB::DocumentError
         |
         |->MongoDB::GridFSError
@@ -437,10 +442,16 @@ a result object.
 Errors indicating failure of a write concern.  The C<result> attribute is a
 result object.
 
+=head2 MongoDB::DecodingError
+
+This error indicates a problem during BSON decoding; it wraps
+the error provided by the underlying BSON encoder.  Note: Encoding errors
+will be thrown as a L</MongoDB::DocumentError>.
+
 =head2 MongoDB::DocumentError
 
 This error indicates a problem with a document to be inserted or replaced into
-the database.
+the database, or used as an update document.
 
 Attributes include:
 
