@@ -20,9 +20,9 @@ package MongoDB::GridFS::File;
 # ABSTRACT: A Mongo GridFS file
 
 use version;
-our $VERSION = 'v0.999.998.4'; # TRIAL
+our $VERSION = 'v0.999.998.6'; # TRIAL
 
-use MongoDB::GridFS;
+use MongoDB::Error;
 use IO::File;
 use Moose;
 use Types::Standard -types;
@@ -72,7 +72,7 @@ sub print {
     my $cursor = $self->_grid->chunks->query({"files_id" => $self->info->{"_id"}})->sort({"n" => 1});
 
     if ( !$cursor->has_next ) {
-        MongoDB::Error->throw(
+        MongoDB::GridFSError->throw(
             sprintf( "GridFS file corrupt: no chunks found for file ID '%s'",
                 $self->info->{_id} )
         );
@@ -104,6 +104,7 @@ sub print {
                 $pos += $len-($offset-$pos);
             }
             else {
+                no warnings 'substr';
                 $fh->print(substr($chunk->{'data'}, $offset-$pos, $length));
                 $written += $length;
                 $pos += $length;
@@ -160,7 +161,7 @@ MongoDB::GridFS::File - A Mongo GridFS file
 
 =head1 VERSION
 
-version v0.999.998.4
+version v0.999.998.6
 
 =head1 SYNOPSIS
 
