@@ -19,13 +19,15 @@ package MongoDB::MongoClient;
 # ABSTRACT: A connection to a MongoDB server or multi-server deployment
 
 use version;
-our $VERSION = 'v0.999.998.5';
+our $VERSION = 'v0.999.998.6'; # TRIAL
 
 use Moose;
 use MongoDB;
-use MongoDB::Cursor;
+use MongoDB::BSON;
 use MongoDB::BSON::Binary;
 use MongoDB::BSON::Regexp;
+use MongoDB::Cursor;
+use MongoDB::DBRef;
 use MongoDB::Error;
 use MongoDB::Op::_Command;
 use MongoDB::ReadPreference;
@@ -55,6 +57,14 @@ use constant {
     NEAREST             => 'nearest',
 };
 
+#--------------------------------------------------------------------------#
+# deprecated global variables
+#--------------------------------------------------------------------------#
+
+# These are used to configure a BSON codec options if none is provided.
+$MongoDB::BSON::looks_like_number = 0;
+$MongoDB::BSON::char = '$';
+
 #--------------------------------------------------------------------------#
 # public attributes
 #
@@ -461,46 +471,24 @@ has auth_mechanism_properties => (
     writer  => '_set_auth_mechanism_properties',
 );
 
-# BSON conversion attributes
+# BSON codec
 
-#pod =attr dt_type
+#pod =attr bson_codec
 #pod
-#pod Sets the type of object which is returned for DateTime fields. The default is
-#pod L<DateTime>. Other acceptable values are L<DateTime::Tiny> and C<undef>. The
-#pod latter will give you the raw epoch value rather than an object.
+#pod An object that provides the C<encode_one> and C<decode_one> methods, such
+#pod as from L<MongoDB::BSON>.
+#pod
+#pod If not provided, one will be generated from deprecated configuration
+#pod options.
 #pod
 #pod =cut
 
-has dt_type => (
-    is      => 'rw',
-    default => 'DateTime'
-);
-
-
-#pod =attr inflate_dbrefs
-#pod
-#pod Controls whether L<DBRef|http://docs.mongodb.org/manual/applications/database-references/#dbref>s
-#pod are automatically inflated into L<MongoDB::DBRef> objects. Defaults to true.
-#pod Set this to C<0> if you don't want to auto-inflate them.
-#pod
-#pod =cut
-
-has inflate_dbrefs => (
-    is      => 'rw',
-    isa     => Bool,
-    default => 1
-);
-
-#pod =attr inflate_regexps
-#pod
-#pod Controls whether regular expressions stored in MongoDB are inflated into L<MongoDB::BSON::Regexp> objects instead of native Perl Regexps. The default is false. This can be dangerous, since the JavaScript regexps used internally by MongoDB are of a different dialect than Perl's. The default for this attribute may become true in future versions of the driver.
-#pod
-#pod =cut
-
-has inflate_regexps => (
-    is      => 'rw',
-    isa     => Bool,
-    default => 0,
+has bson_codec => (
+    is      => 'ro',
+    isa     => BSONCodec,
+    lazy    => 1,
+    builder => '_build_bson_codec',
+    writer  => '_set_bson_codec',
 );
 
 #--------------------------------------------------------------------------#
@@ -534,6 +522,27 @@ has auto_reconnect => (
 );
 
 
+#pod =attr dt_type (DEPRECATED AND READ-ONLY)
+#pod
+#pod Sets the type of object which is returned for DateTime fields. The default is
+#pod L<DateTime>. Other acceptable values are L<DateTime::Tiny> and C<undef>. The
+#pod latter will give you the raw epoch value rather than an object.
+#pod
+#pod This will be used to construct L</bson_codec> object if one is not provided.
+#pod
+#pod As this has a one-time effect, it is now read-only to help you detect
+#pod code that was trying to change after the fact during program execution.
+#pod
+#pod For temporary or localized changes, look into overriding the C<bson_codec>
+#pod object for a database or collection object.
+#pod
+#pod =cut
+
+has dt_type => (
+    is      => 'ro',
+    default => 'DateTime'
+);
+
 #pod =attr find_master (DEPRECATED)
 #pod
 #pod This attribute no longer has any effect.  The driver will always attempt
@@ -648,6 +657,25 @@ sub _build_auth_mechanism_properties {
     };
 }
 
+# skipping op_char unless $MongoDB::BSON::char is not '$' is an optimization
+sub _build_bson_codec {
+    my ($self) = @_;
+    return MongoDB::BSON->new(
+        dbref_callback => sub {
+            my $doc = shift;
+            return MongoDB::DBRef->new(
+                client => $self,
+                ref    => $doc->{'$ref'},
+                id     => $doc->{'$id'},
+                db     => $doc->{'$db'},
+            );
+        },
+        dt_type         => $self->dt_type,
+        prefer_numeric  => $MongoDB::BSON::looks_like_number || 0,
+        ( $MongoDB::BSON::char ne '$' ? ( op_char => $MongoDB::BSON::char ) : () ),
+    );
+}
+
 sub _build__topology {
     my ($self) = @_;
 
@@ -658,6 +686,7 @@ sub _build__topology {
 
     MongoDB::_Topology->new(
         uri                         => $self->_uri,
+        bson_codec                  => $self->bson_codec,
         type                        => $type,
         server_selection_timeout_ms => $self->server_selection_timeout_ms,
         local_threshold_ms          => $self->local_threshold_ms,
@@ -749,6 +778,26 @@ sub BUILD {
         $self->_set_read_preference($rp);
     }
 
+    # Add error handler to codec if user didn't provide their own
+    unless ( $self->bson_codec->error_callback ) {
+        $self->_set_bson_codec(
+            $self->bson_codec->clone(
+                error_callback => sub {
+                    my ($msg, $ref, $op) = @_;
+                    if ( $op =~ /^encode/ ) {
+                        MongoDB::DocumentError->throw(
+                            message => $msg,
+                            document => $ref
+                        );
+                    }
+                    else {
+                        MongoDB::DecodingError->throw($msg);
+                    }
+                },
+            )
+        );
+    }
+
     return;
 }
 
@@ -814,10 +863,11 @@ sub connected {
 sub send_admin_command {
     my ( $self, $command, $read_preference ) = @_;
 
-    my $op   = MongoDB::Op::_Command->new(
-        db_name => 'admin',
-        query => $command,
-        ( $read_preference ? (read_preference => $read_preference) : ()),
+    my $op = MongoDB::Op::_Command->new(
+        db_name    => 'admin',
+        query      => $command,
+        bson_codec => $self->bson_codec,
+        ( $read_preference ? ( read_preference => $read_preference ) : () ),
     );
 
     return $self->send_read_op( $op );
@@ -932,7 +982,8 @@ sub get_database {
     my ( $self, $database_name, $options ) = @_;
     return MongoDB::Database->new(
         read_preference => $self->read_preference,
-        write_concern => $self->_write_concern,
+        write_concern   => $self->_write_concern,
+        bson_codec      => $self->bson_codec,
         ( $options ? %$options : () ),
         # not allowed to be overridden by options
         _client       => $self,
@@ -1082,7 +1133,7 @@ MongoDB::MongoClient - A connection to a MongoDB server or multi-server deployme
 
 =head1 VERSION
 
-version v0.999.998.5
+version v0.999.998.6
 
 =head1 SYNOPSIS
 
@@ -1390,21 +1441,13 @@ available or MONGODB-CR otherwise.
 This is an optional hash reference of authentication mechanism specific properties.
 See L</AUTHENTICATION> for details.
 
-=head2 dt_type
+=head2 bson_codec
 
-Sets the type of object which is returned for DateTime fields. The default is
-L<DateTime>. Other acceptable values are L<DateTime::Tiny> and C<undef>. The
-latter will give you the raw epoch value rather than an object.
+An object that provides the C<encode_one> and C<decode_one> methods, such
+as from L<MongoDB::BSON>.
 
-=head2 inflate_dbrefs
-
-Controls whether L<DBRef|http://docs.mongodb.org/manual/applications/database-references/#dbref>s
-are automatically inflated into L<MongoDB::DBRef> objects. Defaults to true.
-Set this to C<0> if you don't want to auto-inflate them.
-
-=head2 inflate_regexps
-
-Controls whether regular expressions stored in MongoDB are inflated into L<MongoDB::BSON::Regexp> objects instead of native Perl Regexps. The default is false. This can be dangerous, since the JavaScript regexps used internally by MongoDB are of a different dialect than Perl's. The default for this attribute may become true in future versions of the driver.
+If not provided, one will be generated from deprecated configuration
+options.
 
 =head2 auto_connect (DEPRECATED)
 
@@ -1416,6 +1459,20 @@ demand.
 This attribute no longer has any effect.  Connections always reconnect on
 demand.
 
+=head2 dt_type (DEPRECATED AND READ-ONLY)
+
+Sets the type of object which is returned for DateTime fields. The default is
+L<DateTime>. Other acceptable values are L<DateTime::Tiny> and C<undef>. The
+latter will give you the raw epoch value rather than an object.
+
+This will be used to construct L</bson_codec> object if one is not provided.
+
+As this has a one-time effect, it is now read-only to help you detect
+code that was trying to change after the fact during program execution.
+
+For temporary or localized changes, look into overriding the C<bson_codec>
+object for a database or collection object.
+
 =head2 find_master (DEPRECATED)
 
 This attribute no longer has any effect.  The driver will always attempt
