@@ -20,7 +20,7 @@ package MongoDB::Op::_BulkWrite;
 # MongoDB::BulkWriteResult object
 
 use version;
-our $VERSION = 'v0.999.998.4'; # TRIAL
+our $VERSION = 'v0.999.998.6'; # TRIAL
 
 use Moose;
 
@@ -72,7 +72,7 @@ has write_concern => (
 );
 
 # not _WriteOp because we construct our own result objects
-with qw/MongoDB::Role::_CommandOp/;
+with $_ for qw/MongoDB::Role::_CommandOp MongoDB::Role::_UpdatePreEncoder/;
 
 sub execute {
     my ( $self, $link ) = @_;
@@ -133,6 +133,10 @@ sub _execute_write_command_batch {
     while (@left_to_send) {
         my $chunk = shift @left_to_send;
 
+        if ( $cmd eq 'update' ) {
+            $chunk = $self->_pre_encode_update_chunk($link, $chunk);
+        }
+
         my $cmd_doc = [
             $cmd         => $coll_name,
             $op_key      => $chunk,
@@ -141,8 +145,9 @@ sub _execute_write_command_batch {
         ];
 
         my $op = MongoDB::Op::_Command->new(
-            db_name => $db_name,
-            query   => $cmd_doc,
+            db_name    => $db_name,
+            query      => $cmd_doc,
+            bson_codec => $self->bson_codec,
         );
 
         my $cmd_result = try {
@@ -189,6 +194,20 @@ sub _execute_write_command_batch {
     return;
 }
 
+# take array of hash, validate and encode each update doc; since this
+# might be called more than once if chunks are getting split, check if
+# the update doc is already encoded; this also removes the 'is_replace'
+# field that needs to not be in the command sent to the server
+sub _pre_encode_update_chunk {
+    my ( $self, $link, $chunk ) = @_;
+    for ( my $i = 0; $i <= $#$chunk; $i++ ) {
+        next if ref( $chunk->[$i]{u} ) eq 'MongoDB::BSON::Raw';
+        my $is_replace = delete $chunk->[$i]{is_replace};
+        $chunk->[$i]{u} = $self->_pre_encode( $link, $chunk->[$i]{u}, $is_replace );
+    }
+    return $chunk;
+}
+
 sub _split_chunk {
     my ( $self, $link, $chunk, $size ) = @_;
 
@@ -297,6 +316,7 @@ sub _execute_legacy_batch {
                 coll_name     => $self->coll_name,
                 document      => $doc,
                 write_concern => $wc,
+                bson_codec    => $self->bson_codec,
             );
         }
         elsif ( $type eq 'update' ) {
@@ -308,6 +328,8 @@ sub _execute_legacy_batch {
                 multi         => $doc->{multi},
                 upsert        => $doc->{upsert},
                 write_concern => $wc,
+                is_replace    => $doc->{is_replace},
+                bson_codec    => $self->bson_codec,
             );
         }
         elsif ( $type eq 'delete' ) {
@@ -317,6 +339,7 @@ sub _execute_legacy_batch {
                 filter        => $doc->{q},
                 just_one      => !!$doc->{limit},
                 write_concern => $wc,
+                bson_codec    => $self->bson_codec,
             );
         }
 
