@@ -20,7 +20,7 @@ package MongoDB::Op::_GetMore;
 # (after inflation from BSON)
 
 use version;
-our $VERSION = 'v0.999.998.4'; # TRIAL
+our $VERSION = 'v0.999.998.6'; # TRIAL
 
 use Moose;
 
@@ -41,12 +41,6 @@ has client => (
     required => 1,
 );
 
-has bson_codec => (
-    is       => 'ro',
-    isa      => InstanceOf['MongoDB::MongoClient'], # XXX only for now
-    required => 1,
-);
-
 has cursor_id => (
     is       => 'ro',
     isa      => Str,
