@@ -20,13 +20,14 @@ package MongoDB::Op::_InsertOne;
 # MongoDB::InsertOneResult
 
 use version;
-our $VERSION = 'v0.999.998.4'; # TRIAL
+our $VERSION = 'v0.999.998.6'; # TRIAL
 
 use Moose;
 
 use MongoDB::BSON;
 use MongoDB::Error;
 use MongoDB::InsertOneResult;
+use MongoDB::OID;
 use MongoDB::_Protocol;
 use MongoDB::_Types -types;
 use Types::Standard -types;
@@ -46,8 +47,6 @@ has coll_name => (
     required => 1,
 );
 
-# may or may not have _id; caller must get it right. E.g. index creation
-# on legacy mongod does not use _id
 has document => (
     is       => 'ro',
     isa      => IxHash,
@@ -55,14 +54,34 @@ has document => (
     required => 1,
 );
 
+has _doc_id => (
+    is        => 'ro',
+    isa       => Any,
+    writer    => '_set_doc_id',
+);
+
 with qw/MongoDB::Role::_WriteOp/;
 
 sub execute {
     my ( $self, $link ) = @_;
 
+    my $document = $self->document;
+
+    my $id = $self->_set_doc_id(
+        $document->EXISTS('_id') ? $document->FETCH('_id') : MongoDB::OID->new
+    );
+
     # XXX until we have a proper BSON::Raw class, we bless on the fly
-    my $max_size   = $link->max_bson_object_size;
-    my $bson_doc   = MongoDB::BSON::encode_bson( $self->document, 1, $max_size );
+    my $bson_doc = $self->bson_codec->encode_one(
+        $document,
+        {
+            invalid_chars => '.',
+            max_length    => $link->max_bson_object_size,
+            first_key => '_id',
+            first_value => $id,
+        }
+    );
+
     my $insert_doc = bless \$bson_doc, "MongoDB::BSON::Raw";
 
     my $res =
@@ -98,7 +117,7 @@ sub _legacy_op_insert {
 
 sub _parse_cmd {
     my ( $self, $res ) = @_;
-    return ( $res->{ok} ? ( inserted_id => $self->document->FETCH("_id") ) : () );
+    return ( $res->{ok} ? ( inserted_id => $self->_doc_id ) : () );
 }
 
 BEGIN {
