@@ -19,7 +19,7 @@ package MongoDB::Op::_ListIndexes;
 # Encapsulate index list operation; returns array ref of index documents
 
 use version;
-our $VERSION = 'v0.999.998.4'; # TRIAL
+our $VERSION = 'v0.999.998.6'; # TRIAL
 
 use Moose;
 
@@ -51,16 +51,9 @@ has client => (
     required => 1,
 );
 
-has bson_codec => (
-    is       => 'ro',
-    isa      => InstanceOf['MongoDB::MongoClient'], # XXX only for now
-    required => 1,
-);
-
-with qw(
+with $_ for qw(
   MongoDB::Role::_ReadOp
   MongoDB::Role::_CommandCursorOp
-  MongoDB::Role::_EmptyQueryResult
 );
 
 sub execute {
@@ -81,6 +74,7 @@ sub _command_list_indexes {
         db_name         => $self->db_name,
         query           => Tie::IxHash->new( listIndexes => $self->coll_name, cursor => {} ),
         read_preference => $self->read_preference,
+        bson_codec      => $self->bson_codec,
     );
 
     my $res = try {
