@@ -19,7 +19,7 @@ package MongoDB::Role::_CommandCursorOp;
 # MongoDB interface for database commands with cursors
 
 use version;
-our $VERSION = 'v0.999.998.5';
+our $VERSION = 'v0.999.998.6'; # TRIAL
 
 use MongoDB::Error;
 use MongoDB::QueryResult;
@@ -27,7 +27,7 @@ use Moose::Role;
 use Types::Standard -types;
 use namespace::clean -except => 'meta';
 
-requires 'client';
+requires qw/client bson_codec/;
 
 sub _build_result_from_cursor {
     my ( $self, $res ) = @_;
@@ -38,10 +38,26 @@ sub _build_result_from_cursor {
         result  => $res,
       );
 
+    my $qr = MongoDB::QueryResult->new(
+        _client    => $self->client,
+        address    => $res->address,
+        cursor     => $cursor,
+        bson_codec => $self->bson_codec,
+    );
+}
+
+sub _empty_query_result {
+    my ( $self, $link ) = @_;
+
     my $qr = MongoDB::QueryResult->new(
         _client => $self->client,
-        address => $res->address,
-        cursor  => $cursor,
+        address => $link->address,
+        cursor  => {
+            ns         => '',
+            id         => 0,
+            firstBatch => [],
+        },
+        bson_codec => $self->bson_codec,
     );
 }
 
