@@ -19,7 +19,7 @@ package MongoDB::Role::_WriteOp;
 # MongoDB interface for database write operations
 
 use version;
-our $VERSION = 'v0.999.998.4'; # TRIAL
+our $VERSION = 'v0.999.998.6'; # TRIAL
 
 use MongoDB::BSON;
 use MongoDB::CommandResult;
@@ -51,7 +51,7 @@ sub _send_legacy_op_with_gle {
 
     if ( $self->write_concern->is_safe ) {
         my @write_concern = %{ $self->write_concern->as_struct };
-        my $gle = MongoDB::BSON::encode_bson( [ getlasterror => 1, @write_concern ], 0 );
+        my $gle = $self->bson_codec->encode_one( [ getlasterror => 1, @write_concern ] );
         my ( $gle_bson, $request_id ) =
           MongoDB::_Protocol::write_query( $self->db_name . '.$cmd', $gle, undef, 0, -1 );
 
@@ -65,7 +65,7 @@ sub _send_legacy_op_with_gle {
         # errors in the command itself get handled as normal CommandResult
         if ( !$res->{ok} && ( $res->{errmsg} || $res->{'$err'} ) ) {
             return MongoDB::CommandResult->new(
-                result  => $res,
+                output  => $res,
                 address => $link->address,
             );
         }
@@ -83,7 +83,7 @@ sub _send_legacy_op_with_gle {
             MongoDB::DatabaseError->throw(
                 message => $got_error,
                 result => MongoDB::CommandResult->new(
-                    result => $res,
+                    output => $res,
                     address => $link->address,
                 ),
             );
@@ -136,7 +136,7 @@ sub _send_write_command {
         # errors in the command itself get handled as normal CommandResult
         if ( !$res->{ok} && ( $res->{errmsg} || $res->{'$err'} ) ) {
             return MongoDB::CommandResult->new(
-                result  => $res,
+                output => $res,
                 address => $link->address,
             );
         }
