@@ -19,7 +19,7 @@ package MongoDB::WriteConcern;
 # ABSTRACT: Encapsulate and validate a write concern
 
 use version;
-our $VERSION = 'v0.999.998.4'; # TRIAL
+our $VERSION = 'v0.999.998.6'; # TRIAL
 
 use Moose;
 use MongoDB::Error;
@@ -105,7 +105,7 @@ sub _build_as_struct {
 sub BUILD {
     my ($self) = @_;
     if ( ! $self->_w_is_safe && $self->j ) {
-        MongoDB::Error->throw("can't use write concern w=0 with j=" . $self->j );
+        MongoDB::UsageError->throw("can't use write concern w=0 with j=" . $self->j );
     }
     return;
 }
@@ -130,7 +130,7 @@ MongoDB::WriteConcern - Encapsulate and validate a write concern
 
 =head1 VERSION
 
-version v0.999.998.4
+version v0.999.998.6
 
 =head1 SYNOPSIS
 
