@@ -17,9 +17,10 @@
 package MongoDB::_Topology;
 
 use version;
-our $VERSION = 'v0.999.998.5';
+our $VERSION = 'v0.999.998.6'; # TRIAL
 
 use Moose;
+use MongoDB::BSON;
 use MongoDB::Error;
 use MongoDB::Op::_Command;
 use MongoDB::_Link;
@@ -124,6 +125,12 @@ has link_options => (
     default => sub { {} },
 );
 
+has bson_codec => (
+    is       => 'ro',
+    isa      => BSONCodec,
+    default  => sub { MongoDB::BSON->new },
+);
+
 has number_of_seeds => (
     is      => 'ro',
     isa     => Num,
@@ -492,7 +499,7 @@ sub _initialize_link {
     # servers are considered invalid and we throw an error
     if ( $server->type eq any(qw/Standalone Mongos RSPrimary RSSecondary/) ) {
         try {
-            $self->credential->authenticate($link);
+            $self->credential->authenticate($link, $self->bson_codec);
         }
         catch {
             my $err = $_;
@@ -582,8 +589,9 @@ sub _update_topology_from_link {
     my $start_time = [ gettimeofday() ];
     my $is_master = try {
         my $op = MongoDB::Op::_Command->new(
-            db_name => 'admin',
-            query   => [ ismaster => 1 ],
+            db_name    => 'admin',
+            query      => [ ismaster => 1 ],
+            bson_codec => $self->bson_codec,
         );
         $op->execute( $link )->output;
     }
