@@ -19,7 +19,7 @@ package MongoDB::_Types;
 # MongoDB Moose type definitions
 
 use version;
-our $VERSION = 'v0.999.998.4'; # TRIAL
+our $VERSION = 'v0.999.998.6'; # TRIAL
 
 use Type::Library
   -base,
@@ -27,6 +27,7 @@ use Type::Library
   ArrayOfHashRef
   AuthMechanism
   Booleanpm
+  BSONCodec
   ConnectType
   ConnectionStr
   CursorType
@@ -44,10 +45,9 @@ use Type::Library
   NonNegNum
   ReadPrefMode
   ReadPreference
-  ReplaceDoc
   ServerType
+  SingleChar
   TopologyType
-  UpdateDoc
   WriteConcern
 );
 
@@ -83,6 +83,8 @@ enum AuthMechanism,
 
 class_type Booleanpm, { class => 'boolean' };
 
+duck_type BSONCodec, [ qw/encode_one decode_one/ ];
+
 enum ConnectType, [qw/replicaSet direct none/];
 
 declare ConnectionStr, as Str,
@@ -136,6 +138,8 @@ enum ServerType,
     qw/Standalone Mongos PossiblePrimary RSPrimary RSSecondary RSArbiter RSOther RSGhost Unknown/
   ];
 
+declare SingleChar, as Str, where { length $_ eq 1 };
+
 enum TopologyType,
   [qw/Single ReplicaSetNoPrimary ReplicaSetWithPrimary Sharded Unknown/];
 
@@ -181,16 +185,4 @@ coerce ReadPreference, from ArrayRef,
 coerce WriteConcern, from HashRef,
   via { require MongoDB::WriteConcern; MongoDB::WriteConcern->new($_) };
 
-#--------------------------------------------------------------------------#
-# subtypes with inherited coercions
-#--------------------------------------------------------------------------#
-
-declare ReplaceDoc, as IxHash, coercion => 1,
-  where { !$_->Length || substr( $_->Keys(0), 0, 1 ) ne $MongoDB::BSON::char },
-  message { "replacement document ($_) must not contain update operators" };
-
-declare UpdateDoc, as IxHash, coercion => 1,
-  where { $_->Length && substr( $_->Keys(0), 0, 1 ) eq $MongoDB::BSON::char },
-  message { "update document must only contain update operators" };
-
 1;
